=== modified file 'breezy/branch.py'
--- breezy/branch.py	2020-01-20 00:03:54 +0000
+++ breezy/branch.py	2020-02-18 03:11:01 +0000
@@ -1194,7 +1194,8 @@
         if revno < 1 or revno > self.revno():
             raise errors.InvalidRevisionNumber(revno)
 
-    def clone(self, to_controldir, revision_id=None, repository_policy=None):
+    def clone(self, to_controldir, revision_id=None, repository_policy=None,
+              tag_selector=None):
         """Clone this branch into to_controldir preserving all semantic values.
 
         Most API users will want 'create_clone_on_transport', which creates a
@@ -1207,11 +1208,12 @@
         with self.lock_read(), result.lock_write():
             if repository_policy is not None:
                 repository_policy.configure_branch(result)
-            self.copy_content_into(result, revision_id=revision_id)
+            self.copy_content_into(
+                result, revision_id=revision_id, tag_selector=tag_selector)
         return result
 
     def sprout(self, to_controldir, revision_id=None, repository_policy=None,
-               repository=None, lossy=False):
+               repository=None, lossy=False, tag_selector=None):
         """Create a new line of development from the branch, into to_controldir.
 
         to_controldir controls the branch format.
@@ -1228,7 +1230,8 @@
         with self.lock_read(), result.lock_write():
             if repository_policy is not None:
                 repository_policy.configure_branch(result)
-            self.copy_content_into(result, revision_id=revision_id)
+            self.copy_content_into(
+                result, revision_id=revision_id, tag_selector=tag_selector)
             master_url = self.get_bound_location()
             if master_url is None:
                 result.set_parent(self.user_url)
@@ -1261,14 +1264,16 @@
                 revno = 1
         destination.set_last_revision_info(revno, revision_id)
 
-    def copy_content_into(self, destination, revision_id=None):
+    def copy_content_into(self, destination, revision_id=None, tag_selector=None):
         """Copy the content of self into destination.
 
         revision_id: if not None, the revision history in the new branch will
                      be truncated to end with revision_id.
+        tag_selector: Optional callback that receives a tag name
+            and should return a boolean to indicate whether a tag should be copied
         """
         return InterBranch.get(self, destination).copy_content_into(
-            revision_id=revision_id)
+            revision_id=revision_id, tag_selector=tag_selector)
 
     def update_references(self, target):
         if not self._format.supports_reference_locations:
@@ -1313,7 +1318,8 @@
 
     def create_clone_on_transport(self, to_transport, revision_id=None,
                                   stacked_on=None, create_prefix=False,
-                                  use_existing_dir=False, no_tree=None):
+                                  use_existing_dir=False, no_tree=None,
+                                  tag_selector=None):
         """Create a clone of this branch and its bzrdir.
 
         :param to_transport: The transport to clone onto.
@@ -1333,7 +1339,7 @@
         dir_to = self.controldir.clone_on_transport(
             to_transport, revision_id=revision_id, stacked_on=stacked_on,
             create_prefix=create_prefix, use_existing_dir=use_existing_dir,
-            no_tree=no_tree)
+            no_tree=no_tree, tag_selector=tag_selector)
         return dir_to.open_branch()
 
     def create_checkout(self, to_location, revision_id=None,
@@ -2060,7 +2066,7 @@
         raise NotImplementedError(klass._get_branch_formats_to_test)
 
     def pull(self, overwrite=False, stop_revision=None,
-             possible_transports=None, local=False):
+             possible_transports=None, local=False, tag_selector=None):
         """Mirror source into target branch.
 
         The target branch is considered to be 'local', having low latency.
@@ -2070,18 +2076,21 @@
         raise NotImplementedError(self.pull)
 
     def push(self, overwrite=False, stop_revision=None, lossy=False,
-             _override_hook_source_branch=None):
+             _override_hook_source_branch=None, tag_selector=None):
         """Mirror the source branch into the target branch.
 
         The source branch is considered to be 'local', having low latency.
         """
         raise NotImplementedError(self.push)
 
-    def copy_content_into(self, revision_id=None):
+    def copy_content_into(self, revision_id=None, tag_selector=None):
         """Copy the content of source into target
 
-        revision_id: if not None, the revision history in the new branch will
-                     be truncated to end with revision_id.
+        :param revision_id:
+            if not None, the revision history in the new branch will
+            be truncated to end with revision_id.
+        :param tag_selector: Optional callback that can decide
+            to copy or not copy tags.
         """
         raise NotImplementedError(self.copy_content_into)
 
@@ -2128,7 +2137,7 @@
             return format._custom_format
         return format
 
-    def copy_content_into(self, revision_id=None):
+    def copy_content_into(self, revision_id=None, tag_selector=None):
         """Copy the content of source into target
 
         revision_id: if not None, the revision history in the new branch will
@@ -2145,7 +2154,7 @@
                 if parent:
                     self.target.set_parent(parent)
             if self.source._push_should_merge_tags():
-                self.source.tags.merge_to(self.target.tags)
+                self.source.tags.merge_to(self.target.tags, selector=tag_selector)
 
     def fetch(self, stop_revision=None, limit=None, lossy=False):
         if self.target.base == self.source.base:
@@ -2206,7 +2215,8 @@
 
     def pull(self, overwrite=False, stop_revision=None,
              possible_transports=None, run_hooks=True,
-             _override_hook_target=None, local=False):
+             _override_hook_target=None, local=False,
+             tag_selector=None):
         """Pull from source into self, updating my master if any.
 
         :param run_hooks: Private parameter - if false, this branch
@@ -2237,15 +2247,17 @@
             if master_branch:
                 # pull from source into master.
                 master_branch.pull(
-                    self.source, overwrite, stop_revision, run_hooks=False)
+                    self.source, overwrite, stop_revision, run_hooks=False,
+                    tag_selector=tag_selector)
             return self._pull(
                 overwrite, stop_revision, _hook_master=master_branch,
                 run_hooks=run_hooks,
                 _override_hook_target=_override_hook_target,
-                merge_tags_to_master=not source_is_master)
+                merge_tags_to_master=not source_is_master,
+                tag_selector=tag_selector)
 
     def push(self, overwrite=False, stop_revision=None, lossy=False,
-             _override_hook_source_branch=None):
+             _override_hook_source_branch=None, tag_selector=None):
         """See InterBranch.push.
 
         This is the basic concrete implementation of push()
@@ -2277,18 +2289,21 @@
                 with master_branch.lock_write():
                     # push into the master from the source branch.
                     master_inter = InterBranch.get(self.source, master_branch)
-                    master_inter._basic_push(overwrite, stop_revision)
+                    master_inter._basic_push(
+                        overwrite, stop_revision, tag_selector=tag_selector)
                     # and push into the target branch from the source. Note
                     # that we push from the source branch again, because it's
                     # considered the highest bandwidth repository.
-                    result = self._basic_push(overwrite, stop_revision)
+                    result = self._basic_push(
+                        overwrite, stop_revision, tag_selector=tag_selector)
                     result.master_branch = master_branch
                     result.local_branch = self.target
                     _run_hooks()
             else:
                 master_branch = None
                 # no master branch
-                result = self._basic_push(overwrite, stop_revision)
+                result = self._basic_push(
+                    overwrite, stop_revision, tag_selector=tag_selector)
                 # TODO: Why set master_branch and local_branch if there's no
                 # binding?  Maybe cleaner to just leave them unset? -- mbp
                 # 20070504
@@ -2297,7 +2312,7 @@
                 _run_hooks()
             return result
 
-    def _basic_push(self, overwrite, stop_revision):
+    def _basic_push(self, overwrite, stop_revision, tag_selector=None):
         """Basic implementation of push without bound branches or hooks.
 
         Must be called with source read locked and target write locked.
@@ -2316,7 +2331,7 @@
         if self.source._push_should_merge_tags():
             result.tag_updates, result.tag_conflicts = (
                 self.source.tags.merge_to(
-                    self.target.tags, "tags" in overwrite))
+                    self.target.tags, "tags" in overwrite, selector=tag_selector))
         self.update_references()
         result.new_revno, result.new_revid = self.target.last_revision_info()
         return result
@@ -2324,7 +2339,7 @@
     def _pull(self, overwrite=False, stop_revision=None,
               possible_transports=None, _hook_master=None, run_hooks=True,
               _override_hook_target=None, local=False,
-              merge_tags_to_master=True):
+              merge_tags_to_master=True, tag_selector=None):
         """See Branch.pull.
 
         This function is the core worker, used by GenericInterBranch.pull to
@@ -2368,7 +2383,8 @@
             result.tag_updates, result.tag_conflicts = (
                 self.source.tags.merge_to(
                     self.target.tags, "tags" in overwrite,
-                    ignore_master=not merge_tags_to_master))
+                    ignore_master=not merge_tags_to_master,
+                    selector=tag_selector))
             self.update_references()
             result.new_revno, result.new_revid = (
                 self.target.last_revision_info())

=== modified file 'breezy/bzr/branch.py'
--- breezy/bzr/branch.py	2020-02-18 01:32:25 +0000
+++ breezy/bzr/branch.py	2020-02-18 03:11:01 +0000
@@ -1033,7 +1033,7 @@
     def _make_reference_clone_function(format, a_branch):
         """Create a clone() routine for a branch dynamically."""
         def clone(to_bzrdir, revision_id=None,
-                  repository_policy=None):
+                  repository_policy=None, tag_selector=None):
             """See Branch.clone()."""
             return format.initialize(to_bzrdir, target_branch=a_branch)
             # cannot obey revision_id limits when cloning a reference ...

=== modified file 'breezy/bzr/bzrdir.py'
--- breezy/bzr/bzrdir.py	2020-01-19 01:29:22 +0000
+++ breezy/bzr/bzrdir.py	2020-02-18 03:11:01 +0000
@@ -143,7 +143,8 @@
 
     def clone_on_transport(self, transport, revision_id=None,
                            force_new_repo=False, preserve_stacking=False, stacked_on=None,
-                           create_prefix=False, use_existing_dir=True, no_tree=False):
+                           create_prefix=False, use_existing_dir=True, no_tree=False,
+                           tag_selector=None):
         """Clone this bzrdir and its contents to transport verbatim.
 
         :param transport: The transport for the location to produce the clone
@@ -235,7 +236,8 @@
         if local_branch is not None:
             local_branch.clone(
                 result, revision_id=revision_id,
-                repository_policy=repository_policy)
+                repository_policy=repository_policy,
+                tag_selector=tag_selector)
         try:
             # Cheaper to check if the target is not local, than to try making
             # the tree and fail.

=== modified file 'breezy/bzr/remote.py'
--- breezy/bzr/remote.py	2020-02-09 01:16:06 +0000
+++ breezy/bzr/remote.py	2020-02-18 03:11:01 +0000
@@ -2144,11 +2144,11 @@
         self._ensure_real()
         return self._real_repository.get_transaction()
 
-    def clone(self, a_controldir, revision_id=None):
+    def clone(self, a_controldir, revision_id=None, tag_selector=None):
         with self.lock_read():
             dest_repo = self._create_sprouting_repo(
                 a_controldir, shared=self.is_shared())
-            self.copy_content_into(dest_repo, revision_id)
+            self.copy_content_into(dest_repo, revision_id, tag_selector=tag_selector)
             return dest_repo
 
     def make_working_trees(self):
@@ -4033,12 +4033,12 @@
                 source, overwrite=overwrite, stop_revision=stop_revision,
                 _override_hook_target=self, **kwargs)
 
-    def push(self, target, overwrite=False, stop_revision=None, lossy=False):
+    def push(self, target, overwrite=False, stop_revision=None, lossy=False, tag_selector=None):
         with self.lock_read():
             self._ensure_real()
             return self._real_branch.push(
                 target, overwrite=overwrite, stop_revision=stop_revision, lossy=lossy,
-                _override_hook_source_branch=self)
+                _override_hook_source_branch=self, tag_selector=tag_selector)
 
     def peek_lock_mode(self):
         return self._lock_mode

=== modified file 'breezy/controldir.py'
--- breezy/controldir.py	2019-11-06 01:31:41 +0000
+++ breezy/controldir.py	2020-02-18 03:11:01 +0000
@@ -400,7 +400,8 @@
         raise NotImplementedError(self.sprout)
 
     def push_branch(self, source, revision_id=None, overwrite=False,
-                    remember=False, create_prefix=False, lossy=False):
+                    remember=False, create_prefix=False, lossy=False,
+                    tag_selector=None):
         """Push the source branch into this ControlDir."""
         br_to = None
         # If we can open a branch, use its direct repository, otherwise see
@@ -424,7 +425,9 @@
                 # revision
                 revision_id = source.last_revision()
             repository_to.fetch(source.repository, revision_id=revision_id)
-            br_to = source.sprout(self, revision_id=revision_id, lossy=lossy)
+            br_to = source.sprout(
+                self, revision_id=revision_id, lossy=lossy,
+                tag_selector=tag_selector)
             if source.get_push_location() is None or remember:
                 # FIXME: Should be done only if we succeed ? -- vila 2012-01-18
                 source.set_push_location(br_to.base)
@@ -444,17 +447,19 @@
                 tree_to = self.open_workingtree()
             except errors.NotLocalUrl:
                 push_result.branch_push_result = source.push(
-                    br_to, overwrite, stop_revision=revision_id, lossy=lossy)
+                    br_to, overwrite, stop_revision=revision_id, lossy=lossy,
+                    tag_selector=tag_selector)
                 push_result.workingtree_updated = False
             except errors.NoWorkingTree:
                 push_result.branch_push_result = source.push(
-                    br_to, overwrite, stop_revision=revision_id, lossy=lossy)
+                    br_to, overwrite, stop_revision=revision_id, lossy=lossy,
+                    tag_selector=tag_selector)
                 push_result.workingtree_updated = None  # Not applicable
             else:
                 with tree_to.lock_write():
                     push_result.branch_push_result = source.push(
                         tree_to.branch, overwrite, stop_revision=revision_id,
-                        lossy=lossy)
+                        lossy=lossy, tag_selector=tag_selector)
                     tree_to.update()
                 push_result.workingtree_updated = True
             push_result.old_revno = push_result.branch_push_result.old_revno
@@ -493,7 +498,7 @@
         raise NotImplementedError(self.check_conversion_target)
 
     def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
+              preserve_stacking=False, tag_selector=None):
         """Clone this controldir and its contents to url verbatim.
 
         :param url: The url create the clone at.  If url's last component does
@@ -509,11 +514,13 @@
         return self.clone_on_transport(_mod_transport.get_transport(url),
                                        revision_id=revision_id,
                                        force_new_repo=force_new_repo,
-                                       preserve_stacking=preserve_stacking)
+                                       preserve_stacking=preserve_stacking,
+                                       tag_selector=tag_selector)
 
     def clone_on_transport(self, transport, revision_id=None,
                            force_new_repo=False, preserve_stacking=False, stacked_on=None,
-                           create_prefix=False, use_existing_dir=True, no_tree=False):
+                           create_prefix=False, use_existing_dir=True, no_tree=False,
+                           tag_selector=None):
         """Clone this controldir and its contents to transport verbatim.
 
         :param transport: The transport for the location to produce the clone

=== modified file 'breezy/git/branch.py'
--- breezy/git/branch.py	2020-02-18 02:17:28 +0000
+++ breezy/git/branch.py	2020-02-18 03:11:01 +0000
@@ -667,9 +667,9 @@
                 return revision.NULL_REVISION
             return self.lookup_foreign_revision_id(self.head)
 
-    def _basic_push(self, target, overwrite=False, stop_revision=None):
+    def _basic_push(self, target, overwrite=False, stop_revision=None, tag_selector=None):
         return branch.InterBranch.get(self, target)._basic_push(
-            overwrite, stop_revision)
+            overwrite, stop_revision, tag_selector=tag_selector)
 
     def lookup_foreign_revision_id(self, foreign_revid):
         try:
@@ -1043,7 +1043,7 @@
             pass
 
     def _basic_pull(self, stop_revision, overwrite, run_hooks,
-                    _override_hook_target, _hook_master):
+                    _override_hook_target, _hook_master, tag_selector=None):
         if overwrite is True:
             overwrite = set(["history", "tags"])
         elif not overwrite:
@@ -1062,7 +1062,8 @@
             result.new_git_head, remote_refs = self._update_revisions(
                 stop_revision, overwrite=("history" in overwrite))
             tags_ret = self.source.tags.merge_to(
-                self.target.tags, ("tags" in overwrite), ignore_master=True)
+                self.target.tags, ("tags" in overwrite), ignore_master=True,
+                selector=tag_selector)
             if isinstance(tags_ret, tuple):
                 result.tag_updates, result.tag_conflicts = tags_ret
             else:
@@ -1083,7 +1084,7 @@
 
     def pull(self, overwrite=False, stop_revision=None,
              possible_transports=None, _hook_master=None, run_hooks=True,
-             _override_hook_target=None, local=False):
+             _override_hook_target=None, local=False, tag_selector=None):
         """See Branch.pull.
 
         :param _hook_master: Private parameter - set the branch to
@@ -1116,14 +1117,15 @@
                 es.enter_context(master_branch.lock_write())
                 # pull from source into master.
                 master_branch.pull(self.source, overwrite, stop_revision,
-                                   run_hooks=False)
+                                   run_hooks=False, tag_selector=tag_selector)
             else:
                 master_branch = None
             return self._basic_pull(stop_revision, overwrite, run_hooks,
                                     _override_hook_target,
-                                    _hook_master=master_branch)
+                                    _hook_master=master_branch,
+                                    tag_selector=tag_selector)
 
-    def _basic_push(self, overwrite, stop_revision):
+    def _basic_push(self, overwrite, stop_revision, tag_selector=None):
         if overwrite is True:
             overwrite = set(["history", "tags"])
         elif not overwrite:
@@ -1135,7 +1137,8 @@
         result.new_git_head, remote_refs = self._update_revisions(
             stop_revision, overwrite=("history" in overwrite))
         tags_ret = self.source.tags.merge_to(
-            self.target.tags, "tags" in overwrite, ignore_master=True)
+            self.target.tags, "tags" in overwrite, ignore_master=True,
+            selector=tag_selector)
         (result.tag_updates, result.tag_conflicts) = tags_ret
         result.new_revno, result.new_revid = self.target.last_revision_info()
         self.update_references(revid=result.new_revid)
@@ -1164,7 +1167,7 @@
         return (isinstance(source, LocalGitBranch) and
                 isinstance(target, RemoteGitBranch))
 
-    def _basic_push(self, overwrite, stop_revision):
+    def _basic_push(self, overwrite, stop_revision, tag_selector=None):
         result = GitBranchPushResult()
         result.source_branch = self.source
         result.target_branch = self.target
@@ -1189,10 +1192,13 @@
             result.new_revid = stop_revision
             for name, sha in viewitems(
                     self.source.repository._git.refs.as_dict(b"refs/tags")):
+                tag_name = tag_name_to_ref(name)
+                if tag_selector and not tag_selector(tag_name):
+                    continue
                 if sha not in self.source.repository._git:
                     trace.mutter('Ignoring missing SHA: %s', sha)
                     continue
-                refs[tag_name_to_ref(name)] = sha
+                refs[tag_name] = sha
             return refs
         self.target.repository.send_pack(
             get_changed_refs,
@@ -1228,7 +1234,7 @@
         interrepo.fetch_objects(determine_wants, limit=limit, lossy=lossy)
         return _mod_repository.FetchResult()
 
-    def _basic_push(self, overwrite=False, stop_revision=None):
+    def _basic_push(self, overwrite=False, stop_revision=None, tag_selector=None):
         if overwrite is True:
             overwrite = set(["history", "tags"])
         elif not overwrite:
@@ -1244,7 +1250,8 @@
             other_branch=self.source)
         tags_ret = self.source.tags.merge_to(
             self.target.tags,
-            overwrite=("tags" in overwrite))
+            overwrite=("tags" in overwrite),
+            selector=tag_selector)
         if isinstance(tags_ret, tuple):
             (result.tag_updates, result.tag_conflicts) = tags_ret
         else:
@@ -1272,7 +1279,8 @@
         return result.refs, stop_revision
 
     def pull(self, stop_revision=None, overwrite=False,
-             possible_transports=None, run_hooks=True, local=False):
+             possible_transports=None, run_hooks=True, local=False,
+             tag_selector=None):
         # This type of branch can't be bound.
         if local:
             raise errors.LocalRequiresBoundBranch()
@@ -1292,7 +1300,8 @@
                 (result.old_revid if ("history" not in overwrite) else None),
                 other_branch=self.source)
             tags_ret = self.source.tags.merge_to(
-                self.target.tags, overwrite=("tags" in overwrite))
+                self.target.tags, overwrite=("tags" in overwrite),
+                selector=tag_selector)
             if isinstance(tags_ret, tuple):
                 (result.tag_updates, result.tag_conflicts) = tags_ret
             else:
@@ -1331,7 +1340,7 @@
                 isinstance(target, GitBranch))
 
     def _get_new_refs(self, stop_revision=None, fetch_tags=None,
-                      stop_revno=None):
+                      stop_revno=None, tag_selector=None):
         if not self.source.is_locked():
             raise errors.ObjectNotLocked(self.source)
         if stop_revision is None:
@@ -1349,6 +1358,8 @@
             c = self.source.get_config_stack()
             fetch_tags = c.get('branch.fetch_tags')
         for name, revid in viewitems(self.source.tags.get_tag_dict()):
+            if tag_selector and not tag_selector(name):
+                continue
             if self.source.repository.has_revision(revid):
                 ref = tag_name_to_ref(name)
                 if not check_ref_format(ref):
@@ -1360,7 +1371,7 @@
                     refs[ref] = (None, revid)
         return refs, main_ref, (stop_revno, stop_revision)
 
-    def _update_refs(self, result, old_refs, new_refs, overwrite):
+    def _update_refs(self, result, old_refs, new_refs, overwrite, tag_selector):
         mutter("updating refs. old refs: %r, new refs: %r",
                old_refs, new_refs)
         result.tag_updates = {}
@@ -1399,6 +1410,8 @@
                 except ValueError:
                     pass
                 else:
+                    if tag_selector and not tag_selector(tag_name):
+                        continue
                     result.tag_updates[tag_name] = revid
                 ret[ref] = (git_sha, revid)
             else:
@@ -1434,7 +1447,8 @@
             for (old_revid, (new_sha, new_revid)) in revidmap.items()})
 
     def pull(self, overwrite=False, stop_revision=None, local=False,
-             possible_transports=None, run_hooks=True, _stop_revno=None):
+             possible_transports=None, run_hooks=True, _stop_revno=None,
+             tag_selector=None):
         result = GitBranchPullResult()
         result.source_branch = self.source
         result.target_branch = self.target
@@ -1443,10 +1457,10 @@
                 stop_revision, stop_revno=_stop_revno)
 
             def update_refs(old_refs):
-                return self._update_refs(result, old_refs, new_refs, overwrite)
+                return self._update_refs(result, old_refs, new_refs, overwrite, tag_selector)
             try:
                 result.revidmap, old_refs, new_refs = (
-                    self.interrepo.fetch_refs(update_refs, lossy=False))
+                    self.interrepo.fetch_refs(update_refs, lossy=False, overwrite=overwrite))
             except NoPushSupport:
                 raise errors.NoRoundtrippingSupport(self.source, self.target)
             (old_sha1, result.old_revid) = old_refs.get(
@@ -1463,7 +1477,8 @@
         return result
 
     def push(self, overwrite=False, stop_revision=None, lossy=False,
-             _override_hook_source_branch=None, _stop_revno=None):
+             _override_hook_source_branch=None, _stop_revno=None,
+             tag_selector=None):
         result = GitBranchPushResult()
         result.source_branch = self.source
         result.target_branch = self.target
@@ -1471,10 +1486,10 @@
         result.master_branch = result.target_branch
         with self.source.lock_read(), self.target.lock_write():
             new_refs, main_ref, stop_revinfo = self._get_new_refs(
-                stop_revision, stop_revno=_stop_revno)
+                stop_revision, stop_revno=_stop_revno, tag_selector=tag_selector)
 
             def update_refs(old_refs):
-                return self._update_refs(result, old_refs, new_refs, overwrite)
+                return self._update_refs(result, old_refs, new_refs, overwrite, tag_selector)
             try:
                 result.revidmap, old_refs, new_refs = (
                     self.interrepo.fetch_refs(

=== modified file 'breezy/git/dir.py'
--- breezy/git/dir.py	2020-01-18 16:14:28 +0000
+++ breezy/git/dir.py	2020-02-18 03:11:01 +0000
@@ -223,7 +223,8 @@
     def clone_on_transport(self, transport, revision_id=None,
                            force_new_repo=False, preserve_stacking=False,
                            stacked_on=None, create_prefix=False,
-                           use_existing_dir=True, no_tree=False):
+                           use_existing_dir=True, no_tree=False,
+                           tag_selector=None):
         """See ControlDir.clone_on_transport."""
         from ..repository import InterRepository
         from .mapping import default_mapping
@@ -245,7 +246,7 @@
         interrepo = InterRepository.get(source_repo, target_repo)
         if revision_id is not None:
             determine_wants = interrepo.get_determine_wants_revids(
-                [revision_id], include_tags=True)
+                [revision_id], include_tags=True, tag_selector=tag_selector)
         else:
             determine_wants = interrepo.determine_wants_all
         (pack_hint, _, refs) = interrepo.fetch_objects(determine_wants,

=== modified file 'breezy/git/interrepo.py'
--- breezy/git/interrepo.py	2020-01-20 00:26:26 +0000
+++ breezy/git/interrepo.py	2020-02-18 03:11:01 +0000
@@ -81,6 +81,7 @@
     )
 from .refs import (
     is_tag,
+    ref_to_tag_name,
     )
 from .repository import (
     GitRepository,
@@ -409,7 +410,7 @@
     def _target_has_shas(self, shas):
         raise NotImplementedError(self._target_has_shas)
 
-    def get_determine_wants_heads(self, wants, include_tags=False):
+    def get_determine_wants_heads(self, wants, include_tags=False, tag_selector=None):
         wants = set(wants)
 
         def determine_wants(refs):
@@ -422,7 +423,11 @@
                 for k, sha in viewitems(refs):
                     if k.endswith(ANNOTATED_TAG_SUFFIX):
                         continue
-                    if not is_tag(k):
+                    try:
+                        tag_name = ref_to_tag_name(k)
+                    except ValueError:
+                        continue
+                    if tag_selector and not tag_selector(tag_name):
                         continue
                     if sha == ZERO_SHA:
                         continue
@@ -508,14 +513,15 @@
         """
         raise NotImplementedError(self.fetch_objects)
 
-    def get_determine_wants_revids(self, revids, include_tags=False):
+    def get_determine_wants_revids(self, revids, include_tags=False, tag_selector=None):
         wants = set()
         for revid in set(revids):
             if self.target.has_revision(revid):
                 continue
             git_sha, mapping = self.source.lookup_bzr_revision_id(revid)
             wants.add(git_sha)
-        return self.get_determine_wants_heads(wants, include_tags=include_tags)
+        return self.get_determine_wants_heads(
+            wants, include_tags=include_tags, tag_selector=tag_selector)
 
     def fetch(self, revision_id=None, find_ghosts=False,
               mapping=None, fetch_spec=None, include_tags=False, lossy=False):
@@ -688,14 +694,14 @@
         result.refs = wants_recorder.remote_refs
         return result
 
-    def get_determine_wants_revids(self, revids, include_tags=False):
+    def get_determine_wants_revids(self, revids, include_tags=False, tag_selector=None):
         wants = set()
         for revid in set(revids):
             if revid == NULL_REVISION:
                 continue
             git_sha, mapping = self.source.lookup_bzr_revision_id(revid)
             wants.add(git_sha)
-        return self.get_determine_wants_heads(wants, include_tags=include_tags)
+        return self.get_determine_wants_heads(wants, include_tags=include_tags, tag_selector=tag_selector)
 
     def get_determine_wants_branches(self, branches, include_tags=False):
         def determine_wants(refs):

=== modified file 'breezy/git/workingtree.py'
--- breezy/git/workingtree.py	2020-01-19 01:29:22 +0000
+++ breezy/git/workingtree.py	2020-02-18 03:11:01 +0000
@@ -1213,12 +1213,12 @@
 
     def pull(self, source, overwrite=False, stop_revision=None,
              change_reporter=None, possible_transports=None, local=False,
-             show_base=False):
+             show_base=False, tag_selector=None):
         with self.lock_write(), source.lock_read():
             old_revision = self.branch.last_revision()
             count = self.branch.pull(source, overwrite, stop_revision,
                                      possible_transports=possible_transports,
-                                     local=local)
+                                     local=local, tag_selector=tag_selector)
             self._update_git_tree(
                 old_revision=old_revision,
                 new_revision=self.branch.last_revision(),

=== modified file 'breezy/plugins/propose/github.py'
--- breezy/plugins/propose/github.py	2020-01-28 23:33:16 +0000
+++ breezy/plugins/propose/github.py	2020-02-18 03:11:01 +0000
@@ -378,7 +378,7 @@
 
     def publish_derived(self, local_branch, base_branch, name, project=None,
                         owner=None, revision_id=None, overwrite=False,
-                        allow_lossy=True):
+                        allow_lossy=True, tag_selector=None):
         base_owner, base_project, base_branch_name = parse_github_branch_url(base_branch)
         base_repo = self._get_repo(base_owner, base_project)
         if owner is None:
@@ -398,13 +398,14 @@
         try:
             push_result = remote_dir.push_branch(
                 local_branch, revision_id=revision_id, overwrite=overwrite,
-                name=name)
+                name=name, tag_selector=tag_selector)
         except errors.NoRoundtrippingSupport:
             if not allow_lossy:
                 raise
             push_result = remote_dir.push_branch(
                 local_branch, revision_id=revision_id,
-                overwrite=overwrite, name=name, lossy=True)
+                overwrite=overwrite, name=name, lossy=True,
+                tag_selector=tag_selector)
         return push_result.target_branch, github_url_to_bzr_url(
             remote_repo['html_url'], name)
 

=== modified file 'breezy/plugins/propose/gitlabs.py'
--- breezy/plugins/propose/gitlabs.py	2020-01-28 23:33:16 +0000
+++ breezy/plugins/propose/gitlabs.py	2020-02-18 03:11:01 +0000
@@ -418,7 +418,7 @@
 
     def publish_derived(self, local_branch, base_branch, name, project=None,
                         owner=None, revision_id=None, overwrite=False,
-                        allow_lossy=True):
+                        allow_lossy=True, tag_selector=None):
         (host, base_project, base_branch_name) = parse_gitlab_branch_url(base_branch)
         if owner is None:
             owner = self._get_logged_in_username()
@@ -433,13 +433,13 @@
         try:
             push_result = remote_dir.push_branch(
                 local_branch, revision_id=revision_id, overwrite=overwrite,
-                name=name)
+                name=name, tag_selector=tag_selector)
         except errors.NoRoundtrippingSupport:
             if not allow_lossy:
                 raise
             push_result = remote_dir.push_branch(
                 local_branch, revision_id=revision_id, overwrite=overwrite,
-                name=name, lossy=True)
+                name=name, lossy=True, tag_selector=tag_selector)
         public_url = gitlab_url_to_bzr_url(
             target_project['http_url_to_repo'], name)
         return push_result.target_branch, public_url

=== modified file 'breezy/plugins/propose/launchpad.py'
--- breezy/plugins/propose/launchpad.py	2020-01-28 23:33:16 +0000
+++ breezy/plugins/propose/launchpad.py	2020-02-18 03:11:01 +0000
@@ -260,7 +260,8 @@
         return "~%s/%s" % (owner, project)
 
     def _publish_git(self, local_branch, base_path, name, owner, project=None,
-                     revision_id=None, overwrite=False, allow_lossy=True):
+                     revision_id=None, overwrite=False, allow_lossy=True,
+                     tag_selector=None):
         to_path = self._get_derived_git_path(base_path, owner, project)
         to_transport = get_transport("git+ssh://git.launchpad.net/" + to_path)
         try:
@@ -272,21 +273,23 @@
         if dir_to is None:
             try:
                 br_to = local_branch.create_clone_on_transport(
-                    to_transport, revision_id=revision_id, name=name)
+                    to_transport, revision_id=revision_id, name=name,
+                    tag_selector=tag_selector)
             except errors.NoRoundtrippingSupport:
                 br_to = local_branch.create_clone_on_transport(
                     to_transport, revision_id=revision_id, name=name,
-                    lossy=True)
+                    lossy=True, tag_selector=tag_selector)
         else:
             try:
                 dir_to = dir_to.push_branch(
-                    local_branch, revision_id, overwrite=overwrite, name=name)
+                    local_branch, revision_id, overwrite=overwrite, name=name,
+                    tag_selector=tag_selector)
             except errors.NoRoundtrippingSupport:
                 if not allow_lossy:
                     raise
                 dir_to = dir_to.push_branch(
                     local_branch, revision_id, overwrite=overwrite, name=name,
-                    lossy=True)
+                    lossy=True, tag_selector=tag_selector)
             br_to = dir_to.target_branch
         return br_to, (
             "https://git.launchpad.net/%s/+ref/%s" % (to_path, name))
@@ -312,7 +315,7 @@
 
     def _publish_bzr(self, local_branch, base_branch, name, owner,
                      project=None, revision_id=None, overwrite=False,
-                     allow_lossy=True):
+                     allow_lossy=True, tag_selector=None):
         to_path = self._get_derived_bzr_path(base_branch, name, owner, project)
         to_transport = get_transport("lp:" + to_path)
         try:
@@ -323,10 +326,11 @@
 
         if dir_to is None:
             br_to = local_branch.create_clone_on_transport(
-                to_transport, revision_id=revision_id)
+                to_transport, revision_id=revision_id, tag_selector=tag_selector)
         else:
             br_to = dir_to.push_branch(
-                local_branch, revision_id, overwrite=overwrite).target_branch
+                local_branch, revision_id, overwrite=overwrite,
+                tag_selector=tag_selector).target_branch
         return br_to, ("https://code.launchpad.net/" + to_path)
 
     def _split_url(self, url):
@@ -343,7 +347,7 @@
 
     def publish_derived(self, local_branch, base_branch, name, project=None,
                         owner=None, revision_id=None, overwrite=False,
-                        allow_lossy=True):
+                        allow_lossy=True, tag_selector=None):
         """Publish a branch to the site, derived from base_branch.
 
         :param base_branch: branch to derive the new branch from
@@ -362,12 +366,12 @@
             return self._publish_bzr(
                 local_branch, base_branch, name, project=project, owner=owner,
                 revision_id=revision_id, overwrite=overwrite,
-                allow_lossy=allow_lossy)
+                allow_lossy=allow_lossy, tag_selector=tag_selector)
         elif base_vcs == 'git':
             return self._publish_git(
                 local_branch, base_path, name, project=project, owner=owner,
                 revision_id=revision_id, overwrite=overwrite,
-                allow_lossy=allow_lossy)
+                allow_lossy=allow_lossy, tag_selector=tag_selector)
         else:
             raise AssertionError('not a valid Launchpad URL')
 

=== modified file 'breezy/plugins/weave_fmt/bzrdir.py'
--- breezy/plugins/weave_fmt/bzrdir.py	2020-01-31 10:39:02 +0000
+++ breezy/plugins/weave_fmt/bzrdir.py	2020-02-18 03:11:01 +0000
@@ -747,7 +747,7 @@
         return self._format.__class__()
 
     def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
+              preserve_stacking=False, tag_selector=tag_selector):
         """See ControlDir.clone().
 
         force_new_repo has no effect, since this family of formats always
@@ -759,7 +759,7 @@
         result = self._format._initialize_for_clone(url)
         self.open_repository().clone(result, revision_id=revision_id)
         from_branch = self.open_branch()
-        from_branch.clone(result, revision_id=revision_id)
+        from_branch.clone(result, revision_id=revision_id, tag_selector=tag_selector)
         try:
             tree = self.open_workingtree()
         except errors.NotLocalUrl:

=== modified file 'breezy/propose.py'
--- breezy/propose.py	2020-01-28 23:33:16 +0000
+++ breezy/propose.py	2020-02-18 03:11:01 +0000
@@ -239,7 +239,7 @@
 
     def publish_derived(self, new_branch, base_branch, name, project=None,
                         owner=None, revision_id=None, overwrite=False,
-                        allow_lossy=True):
+                        allow_lossy=True, tag_selector=None):
         """Publish a branch to the site, derived from base_branch.
 
         :param base_branch: branch to derive the new branch from

=== modified file 'breezy/tests/test_foreign.py'
--- breezy/tests/test_foreign.py	2018-11-25 20:44:56 +0000
+++ breezy/tests/test_foreign.py	2020-02-18 03:11:01 +0000
@@ -171,7 +171,7 @@
     def is_compatible(source, target):
         return isinstance(target, DummyForeignVcsBranch)
 
-    def push(self, overwrite=False, stop_revision=None, lossy=False):
+    def push(self, overwrite=False, stop_revision=None, lossy=False, tag_selector=None):
         if not lossy:
             raise errors.NoRoundtrippingSupport(self.source, self.target)
         result = branch.BranchPushResult()

=== modified file 'breezy/workingtree.py'
--- breezy/workingtree.py	2020-01-30 17:54:42 +0000
+++ breezy/workingtree.py	2020-02-18 03:11:01 +0000
@@ -821,13 +821,13 @@
 
     def pull(self, source, overwrite=False, stop_revision=None,
              change_reporter=None, possible_transports=None, local=False,
-             show_base=False):
+             show_base=False, tag_selector=None):
         with self.lock_write(), source.lock_read():
             old_revision_info = self.branch.last_revision_info()
             basis_tree = self.basis_tree()
             count = self.branch.pull(source, overwrite, stop_revision,
                                      possible_transports=possible_transports,
-                                     local=local)
+                                     local=local, tag_selector=tag_selector)
             new_revision_info = self.branch.last_revision_info()
             if new_revision_info != old_revision_info:
                 repository = self.branch.repository

